@@ -11,6 +11,7 @@
 # limitations under the License.
 
 import itertools
+import os
 
 from oslo_config import cfg
 from oslo_config import types
@@ -19,17 +20,35 @@ from kolla.version import version_info as version
 
 
 BASE_OS_DISTRO = ['centos', 'rhel', 'ubuntu', 'oraclelinux', 'debian']
+BASE_ARCH = ['x86_64', 'ppc64le', 'aarch64']
+DEFAULT_BASE_TAGS = {
+    'centos': '7',
+    'rhel': '7',
+    'oraclelinux': '7-slim',
+    'debian': 'stretch',
+    'ubuntu': '16.04',
+}
 DISTRO_RELEASE = {
     'centos': '7',
     'rhel': '7',
     'oraclelinux': '7',
-    'debian': '8',
+    'debian': 'stretch',
     'ubuntu': '16.04',
 }
-DELOREAN = ("http://buildlogs.centos.org/centos/7/cloud/x86_64/"
-            "rdo-trunk-master-tested/delorean.repo")
-# TODO(pbourke): update to buildlogs.centos.org once this moves
-DELOREAN_DEPS = "http://trunk.rdoproject.org/centos7/delorean-deps.repo"
+
+# This is noarch repository so we will use it on all architectures
+DELOREAN = \
+    "https://trunk.rdoproject.org/centos7/current-passed-ci/delorean.repo"
+
+# TODO(hrw): with move to Pike+1 we need to make sure that aarch64 repo
+#            gets updated (docker/base/aarch64-cbs.repo file)
+#            there is ongoing work to sort that out
+DELOREAN_DEPS = {
+    'x86_64': "https://trunk.rdoproject.org/centos7/delorean-deps.repo",
+    'aarch64': "",
+    'ppc64le': ""
+}
+
 INSTALL_TYPE_CHOICES = ['binary', 'source', 'rdo', 'rhos']
 
 TARBALLS_BASE = "http://tarballs.openstack.org"
@@ -49,9 +68,12 @@ _PROFILE_OPTS = [
                     'mariadb',
                     'memcached',
                     'mongodb',
+                    'opendaylight',
                     'openvswitch',
+                    'qdrouterd',
                     'rabbitmq',
                     'redis',
+                    'skydive',
                     'tgtd',
                 ],
                 help='Infra images'),
@@ -65,16 +87,19 @@ _PROFILE_OPTS = [
                     'iscsi',
                     'keystone',
                     'neutron',
-                    'nova',
+                    'nova-',
                     'swift',
                 ],
                 help='Main images'),
     cfg.ListOpt('aux',
                 default=[
                     'aodh',
+                    'blazar',
                     'cloudkitty',
                     'congress',
                     'designate',
+                    'dragonflow',
+                    'ec2-api',
                     'freezer',
                     'gnocchi',
                     'influxdb',
@@ -87,6 +112,7 @@ _PROFILE_OPTS = [
                     'mistral',
                     'monasca',
                     'murano',
+                    'novajoin',
                     'octavia',
                     'panko',
                     'rally',
@@ -119,7 +145,7 @@ _PROFILE_OPTS = [
                     'mariadb',
                     'memcached',
                     'neutron',
-                    'nova',
+                    'nova-',
                     'openvswitch',
                     'rabbitmq',
                 ],
@@ -138,13 +164,15 @@ _PROFILE_OPTS = [
                     'mariadb',
                     'memcached',
                     'neutron',
-                    'nova',
+                    'nova-',
                     'openvswitch',
                     'rabbitmq',
                 ],
                 help='Gate images')
 ]
 
+hostarch = os.uname()[4]
+
 _CLI_OPTS = [
     cfg.StrOpt('base', short='b', default='centos',
                choices=BASE_OS_DISTRO,
@@ -153,9 +181,18 @@ _CLI_OPTS = [
     cfg.StrOpt('base-tag', default='latest',
                help='The base distro image tag'),
     cfg.StrOpt('base-image',
-               help='The base image name. Default is the same with base'),
+               help='The base image name. Default is the same with base. '
+                    'For non-x86 architectures use full name like '
+                    '"aarch64/debian".'),
+    cfg.StrOpt('base-arch', default=hostarch,
+               choices=BASE_ARCH,
+               help='The base architecture. Default is same as host'),
     cfg.BoolOpt('debug', short='d', default=False,
                 help='Turn on debugging log level'),
+    cfg.BoolOpt('skip-parents', default=False,
+                help='Do not rebuild parents of matched images'),
+    cfg.BoolOpt('skip-existing', default=False,
+                help='Do not rebuild images present in the docker cache'),
     cfg.DictOpt('build-args',
                 help='Set docker build time variables'),
     cfg.BoolOpt('keep', default=False,
@@ -167,8 +204,7 @@ _CLI_OPTS = [
     cfg.StrOpt('namespace', short='n', default='kolla',
                help='The Docker namespace name'),
     cfg.BoolOpt('cache', default=True,
-                help='Use the Docker cache when building',
-                ),
+                help='Use the Docker cache when building'),
     cfg.MultiOpt('profile', types.String(), short='p',
                  help=('Build a pre-defined set of images, see [profiles]'
                        ' section in config. The default profiles are:'
@@ -180,7 +216,7 @@ _CLI_OPTS = [
     cfg.IntOpt('push-threads', default=1, min=1,
                help=('The number of threads to user while pushing'
                      ' Images. Note: Docker can not handle threading'
-                     ' push properly.')),
+                     ' push properly')),
     cfg.IntOpt('retries', short='r', default=3, min=0,
                help='The number of times to retry while building'),
     cfg.MultiOpt('regex', types.String(), positional=True,
@@ -194,7 +230,7 @@ _CLI_OPTS = [
                      ' dependency in Graphviz dot format')),
     cfg.StrOpt('format', short='f', default='json',
                choices=['json', 'none'],
-               help=('Format to write the final results in')),
+               help='Format to write the final results in'),
     cfg.StrOpt('tarballs-base', default=TARBALLS_BASE,
                help='Base url to OpenStack tarballs'),
     cfg.StrOpt('type', short='t', default='binary',
@@ -205,18 +241,23 @@ _CLI_OPTS = [
     cfg.IntOpt('threads', short='T', default=8, min=1,
                help=('The number of threads to use while building.'
                      ' (Note: setting to one will allow real time'
-                     ' logging.)')),
+                     ' logging)')),
     cfg.StrOpt('tag', default=version.cached_version_string(),
                help='The Docker tag'),
     cfg.BoolOpt('template-only', default=False,
-                help=("Don't build images. Generate Dockerfile only")),
+                help="Don't build images. Generate Dockerfile only"),
     cfg.IntOpt('timeout', default=120,
                help='Time in seconds after which any operation times out'),
     cfg.MultiOpt('template-override', types.String(),
                  help='Path to template override file'),
+    cfg.MultiOpt('docker-dir', types.String(),
+                 help='Path to additional docker file template directory',
+                 short='D', default=[]),
     cfg.StrOpt('logs-dir', help='Path to logs directory'),
     cfg.BoolOpt('pull', default=True,
-                help='Attempt to pull a newer version of the base image.'),
+                help='Attempt to pull a newer version of the base image'),
+    cfg.StrOpt('work-dir', help=('Path to be used as working directory.'
+                                 'By default, a temporary dir is created')),
 ]
 
 _BASE_OPTS = [
@@ -235,304 +276,383 @@ SOURCES = {
     'openstack-base': {
         'type': 'url',
         'location': ('$tarballs_base/requirements/'
-                     'requirements-stable-ocata.tar.gz')},
+                     'requirements-stable-pike.tar.gz')},
     'aodh-base': {
         'type': 'url',
         'location': ('$tarballs_base/aodh/'
-                     'aodh-4.0.3.tar.gz')},
+                     'aodh-5.0.0.tar.gz')},
     'barbican-base': {
         'type': 'url',
         'location': ('$tarballs_base/barbican/'
-                     'barbican-4.0.0.tar.gz')},
+                     'barbican-5.0.0.tar.gz')},
     'bifrost-base': {
         'type': 'url',
         'location': ('$tarballs_base/bifrost/'
-                     'bifrost-3.1.1.tar.gz')},
+                     'bifrost-4.0.0.tar.gz')},
+    'blazar-base': {
+        'type': 'url',
+        'location': ('$tarballs_base/blazar/'
+                     'blazar-0.3.0.tar.gz')},
     'ceilometer-base': {
         'type': 'url',
         'location': ('$tarballs_base/ceilometer/'
-                     'ceilometer-8.1.5.tar.gz')},
+                     'ceilometer-9.0.1.tar.gz')},
     'ceilometer-base-plugin-panko': {
         'type': 'url',
         'location': ('$tarballs_base/panko/'
-                     'panko-2.0.3.tar.gz')},
+                     'panko-3.0.0.tar.gz')},
     'cinder-base': {
         'type': 'url',
         'location': ('$tarballs_base/cinder/'
-                     'cinder-10.0.7.tar.gz')},
+                     'cinder-11.0.0.tar.gz')},
     'congress-base': {
         'type': 'url',
         'location': ('$tarballs_base/congress/'
-                     'congress-5.0.1.tar.gz')},
+                     'congress-6.0.0.tar.gz')},
     'cloudkitty-base': {
         'type': 'url',
         'location': ('$tarballs_base/cloudkitty/'
-                     'cloudkitty-5.0.1.tar.gz')},
+                     'cloudkitty-6.0.0.tar.gz')},
     'designate-base': {
         'type': 'url',
         'location': ('$tarballs_base/designate/'
-                     'designate-4.0.0.tar.gz')},
+                     'designate-5.0.0.tar.gz')},
+    'dragonflow-base': {
+        'type': 'url',
+        'location': ('$tarballs_base/dragonflow/'
+                     'dragonflow-4.0.0.tar.gz')},
+    'ec2-api': {
+        'type': 'url',
+        'location': ('$tarballs_base/ec2-api/'
+                     'ec2-api-5.0.0.tar.gz')},
     'freezer-api': {
         'type': 'url',
         'location': ('$tarballs_base/freezer-api/'
-                     'freezer-api-4.0.1.tar.gz')},
+                     'freezer-api-5.0.0.tar.gz')},
     'freezer-base': {
         'type': 'url',
         'location': ('$tarballs_base/freezer/'
-                     'freezer-4.0.1.tar.gz')},
+                     'freezer-5.0.0.tar.gz')},
     'glance-base': {
         'type': 'url',
         'location': ('$tarballs_base/glance/'
-                     'glance-14.0.1.tar.gz')},
+                     'glance-15.0.0.tar.gz')},
     'gnocchi-base': {
-        'type': 'url',
-        'location': ('$tarballs_base/gnocchi/'
-                     'gnocchi-3.1.2.tar.gz')},
+        'type': 'git',
+        'reference': '4.0.3',
+        'location': ('https://github.com/gnocchixyz/'
+                     'gnocchi.git')},
     'heat-base': {
         'type': 'url',
         'location': ('$tarballs_base/heat/'
-                     'heat-8.0.6.tar.gz')},
+                     'heat-9.0.0.tar.gz')},
     'horizon': {
         'type': 'url',
         'location': ('$tarballs_base/horizon/'
-                     'horizon-11.0.4.tar.gz')},
+                     'horizon-12.0.0.tar.gz')},
     'horizon-plugin-cloudkitty-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/cloudkitty-dashboard/'
-                     'cloudkitty-dashboard-5.0.0.tar.gz')},
+                     'cloudkitty-dashboard-6.0.0.tar.gz')},
     'horizon-plugin-designate-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/designate-dashboard/'
-                     'designate-dashboard-4.0.0.tar.gz')},
+                     'designate-dashboard-5.0.0.tar.gz')},
+    'horizon-plugin-fwaas-dashboard': {
+        'type': 'url',
+        'location': ('$tarballs_base/neutron-fwaas-dashboard/'
+                     'neutron-fwaas-dashboard-1.0.0.tar.gz')},
     'horizon-plugin-freezer-web-ui': {
         'type': 'url',
         'location': ('$tarballs_base/freezer-web-ui/'
-                     'freezer-web-ui-4.0.1.tar.gz')},
+                     'freezer-web-ui-5.0.0.tar.gz')},
     'horizon-plugin-ironic-ui': {
         'type': 'url',
         'location': ('$tarballs_base/ironic-ui/'
-                     'ironic-ui-2.2.2.tar.gz')},
+                     'ironic-ui-3.0.1.tar.gz')},
     'horizon-plugin-karbor-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/karbor-dashboard/'
-                     'karbor-dashboard-0.1.2.tar.gz')},
+                     'karbor-dashboard-0.4.0.tar.gz')},
     'horizon-plugin-magnum-ui': {
         'type': 'url',
         'location': ('$tarballs_base/magnum-ui/'
-                     'magnum-ui-2.2.0.tar.gz')},
+                     'magnum-ui-3.0.0.tar.gz')},
     'horizon-plugin-manila-ui': {
         'type': 'url',
         'location': ('$tarballs_base/manila-ui/'
-                     'manila-ui-2.7.1.tar.gz')},
-    'horizon-plugin-mistral-ui': {
+                     'manila-ui-2.10.2.tar.gz')},
+    'horizon-plugin-mistral-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/mistral-dashboard/'
-                     'mistral-dashboard-4.0.0.tar.gz')},
+                     'mistral-dashboard-5.1.0.tar.gz')},
     'horizon-plugin-monasca-ui': {
         'type': 'url',
         'location': ('$tarballs_base/monasca-ui/'
-                     'monasca-ui-1.5.4.tar.gz')},
+                     'monasca-ui-1.8.0.tar.gz')},
     'horizon-plugin-murano-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/murano-dashboard/'
-                     'murano-dashboard-3.2.0.tar.gz')},
+                     'murano-dashboard-4.0.0.tar.gz')},
     'horizon-plugin-neutron-lbaas-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/neutron-lbaas-dashboard/'
-                     'neutron-lbaas-dashboard-2.0.0.tar.gz')},
+                     'neutron-lbaas-dashboard-3.0.1.tar.gz')},
     'horizon-plugin-sahara-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/sahara-dashboard/'
-                     'sahara-dashboard-6.0.3.tar.gz')},
+                     'sahara-dashboard-7.0.0.tar.gz')},
     'horizon-plugin-searchlight-ui': {
         'type': 'url',
         'location': ('$tarballs_base/searchlight-ui/'
-                     'searchlight-ui-2.0.0.tar.gz')},
+                     'searchlight-ui-3.0.0.tar.gz')},
     'horizon-plugin-senlin-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/senlin-dashboard/'
-                     'senlin-dashboard-0.6.0.tar.gz')},
+                     'senlin-dashboard-0.7.0.tar.gz')},
     'horizon-plugin-solum-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/solum-dashboard/'
-                     'solum-dashboard-2.0.0.tar.gz')},
+                     'solum-dashboard-2.2.0.tar.gz')},
     'horizon-plugin-tacker-dashboard': {
         'type': 'url',
         'location': ('http://tarballs.openstack.org/tacker-horizon/'
-                     'tacker-horizon-0.8.0.tar.gz')},
+                     'tacker-horizon-0.10.0.tar.gz')},
     'horizon-plugin-trove-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/trove-dashboard/'
-                     'trove-dashboard-8.0.0.tar.gz')},
+                     'trove-dashboard-9.0.0.tar.gz')},
     'horizon-plugin-watcher-dashboard': {
         'type': 'url',
         'location': ('$tarballs_base/watcher-dashboard/'
-                     'watcher-dashboard-1.0.0.tar.gz')},
+                     'watcher-dashboard-1.4.0.tar.gz')},
     'horizon-plugin-zaqar-ui': {
         'type': 'url',
         'location': ('$tarballs_base/zaqar-ui/'
-                     'zaqar-ui-2.0.0.tar.gz')},
+                     'zaqar-ui-3.0.0.tar.gz')},
+    'horizon-plugin-zun-ui': {
+        'type': 'url',
+        'location': ('$tarballs_base/zun-ui/'
+                     'zun-ui-0.2.0.tar.gz')},
     'ironic-base': {
         'type': 'url',
         'location': ('$tarballs_base/ironic/'
-                     'ironic-7.0.5.tar.gz')},
+                     'ironic-9.1.1.tar.gz')},
     'ironic-inspector': {
         'type': 'url',
         'location': ('$tarballs_base/ironic-inspector/'
-                     'ironic-inspector-5.0.2.tar.gz')},
+                     'ironic-inspector-6.0.0.tar.gz')},
     'karbor-base': {
         'type': 'url',
         'location': ('$tarballs_base/karbor/'
-                     'karbor-0.2.0.tar.gz')},
+                     'karbor-0.5.1.tar.gz')},
     'keystone-base': {
         'type': 'url',
         'location': ('$tarballs_base/keystone/'
-                     'keystone-11.0.3.tar.gz')},
+                     'keystone-12.0.0.tar.gz')},
     'kuryr-base': {
         'type': 'url',
         'location': ('$tarballs_base/kuryr/'
-                     'kuryr-lib-0.4.0.tar.gz')},
+                     'kuryr-lib-0.6.0.tar.gz')},
     'kuryr-libnetwork': {
         'type': 'url',
         'location': ('$tarballs_base/kuryr-libnetwork/'
-                     'kuryr-libnetwork-0.1.0.tar.gz')},
+                     'kuryr-libnetwork-0.2.0.tar.gz')},
     'magnum-base': {
         'type': 'url',
         'location': ('$tarballs_base/magnum/'
-                     'magnum-4.1.4.tar.gz')},
+                     'magnum-5.0.1.tar.gz')},
     'manila-base': {
         'type': 'url',
         'location': ('$tarballs_base/manila/'
-                     'manila-4.0.1.tar.gz')},
+                     'manila-5.0.1.tar.gz')},
     'mistral-base': {
         'type': 'url',
         'location': ('$tarballs_base/mistral/'
-                     'mistral-4.0.2.tar.gz')},
+                     'mistral-5.1.0.tar.gz')},
+    'mistral-base-plugin-tacker': {
+        'type': 'url',
+        'location': ('$tarballs_base/tacker/'
+                     'tacker-0.8.0.tar.gz')},
     'monasca-api': {
         'type': 'url',
         'location': ('$tarballs_base/monasca-api/'
-                     'monasca-api-1.7.0.tar.gz')},
+                     'monasca-api-2.2.0.tar.gz')},
     'monasca-log-api': {
         'type': 'url',
         'location': ('$tarballs_base/monasca-log-api/'
-                     'monasca-log-api-1.4.2.tar.gz')},
+                     'monasca-log-api-2.3.0.tar.gz')},
     'monasca-notification': {
         'type': 'url',
         'location': ('$tarballs_base/monasca-notification/'
-                     'monasca-notification-1.7.0.tar.gz')},
+                     'monasca-notification-1.10.1.tar.gz')},
     'monasca-persister': {
         'type': 'url',
         'location': ('$tarballs_base/monasca-persister/'
-                     'monasca-persister-1.4.0.tar.gz')},
+                     'monasca-persister-1.7.0.tar.gz')},
     'monasca-statsd': {
         'type': 'url',
         'location': ('$tarballs_base/monasca-statsd/'
-                     'monasca-statsd-1.5.0.tar.gz')},
+                     'monasca-statsd-1.7.0.tar.gz')},
     'murano-base': {
         'type': 'url',
         'location': ('$tarballs_base/murano/'
-                     'murano-3.2.0.tar.gz')},
+                     'murano-4.0.0.tar.gz')},
     'neutron-base': {
         'type': 'url',
         'location': ('$tarballs_base/neutron/'
-                     'neutron-10.0.7.tar.gz')},
+                     'neutron-11.0.1.tar.gz')},
     'neutron-base-plugin-neutron-fwaas': {
         'type': 'url',
         'location': ('$tarballs_base/neutron-fwaas/'
-                     'neutron-fwaas-10.1.0.tar.gz')},
+                     'neutron-fwaas-11.0.1.tar.gz')},
     'neutron-base-plugin-networking-generic-switch': {
         'type': 'url',
         'location': ('$tarballs_base/networking-generic-switch/'
-                     'networking-generic-switch-stable-ocata.tar.gz')},
+                     'networking-generic-switch-0.4.0.tar.gz')},
+    'neutron-base-plugin-networking-sfc': {
+        'type': 'url',
+        'location': ('$tarballs_base/networking-sfc/'
+                     'networking-sfc-5.0.0.tar.gz')},
+    'neutron-base-plugin-vmware-nsx': {
+        'type': 'url',
+        'location': ('$tarballs_base/vmware-nsx/'
+                     'vmware-nsx-11.0.0.tar.gz')},
+    'neutron-bgp-dragent': {
+        'type': 'url',
+        'location': ('$tarballs_base/neutron-dynamic-routing/'
+                     'neutron-dynamic-routing-11.0.0.tar.gz')},
     'neutron-lbaas-agent': {
         'type': 'url',
         'location': ('$tarballs_base/neutron-lbaas/'
-                     'neutron-lbaas-10.0.1.tar.gz')},
+                     'neutron-lbaas-11.0.1.tar.gz')},
+    'neutron-server-opendaylight-plugin-networking-odl': {
+        'type': 'url',
+        'location': ('$tarballs_base/networking-odl/'
+                     'networking-odl-11.0.0.tar.gz')},
+    'neutron-server-opendaylight-plugin-networking-bgpvpn': {
+        'type': 'url',
+        'location': ('$tarballs_base/networking-bgpvpn/'
+                     'networking-bgpvpn-7.0.0.tar.gz')},
+    'neutron-server-opendaylight-plugin-networking-l2gw': {
+        'type': 'url',
+        'location': ('$tarballs_base/networking-l2gw/'
+                     'networking-l2gw-11.0.0.tar.gz')},
+    'neutron-server-opendaylight-plugin-networking-sfc': {
+        'type': 'url',
+        'location': ('$tarballs_base/networking-sfc/'
+                     'networking-sfc-5.0.0.tar.gz')},
+    'neutron-server-plugin-neutron-dynamic-routing': {
+        'type': 'url',
+        'location': ('$tarballs_base/neutron-dynamic-routing/'
+                     'neutron-dynamic-routing-11.0.0.tar.gz')},
     'neutron-server-plugin-neutron-lbaas': {
         'type': 'url',
         'location': ('$tarballs_base/neutron-lbaas/'
-                     'neutron-lbaas-10.0.1.tar.gz')},
-    'neutron-sfc-agent': {
+                     'neutron-lbaas-11.0.1.tar.gz')},
+    'neutron-server-plugin-vmware-nsxlib': {
         'type': 'url',
-        'location': ('$tarballs_base/networking-sfc/'
-                     'networking-sfc-4.0.0.tar.gz')},
+        'location': ('$tarballs_base/vmware-nsxlib/'
+                     'vmware-nsxlib-11.1.2.tar.gz')},
     'neutron-server-plugin-vpnaas-agent': {
         'type': 'url',
         'location': ('$tarballs_base/neutron-vpnaas/'
-                     'neutron-vpnaas-10.0.0.tar.gz')},
+                     'neutron-vpnaas-11.0.0.tar.gz')},
     'neutron-vpnaas-agent': {
         'type': 'url',
         'location': ('$tarballs_base/neutron-vpnaas/'
-                     'neutron-vpnaas-10.0.0.tar.gz')},
+                     'neutron-vpnaas-11.0.0.tar.gz')},
+    'neutron-server-ovn-plugin-networking-ovn': {
+        'type': 'url',
+        'location': ('$tarballs_base/networking-ovn/'
+                     'networking-ovn-3.0.0.tar.gz')},
     'nova-base': {
         'type': 'url',
         'location': ('$tarballs_base/nova/'
-                     'nova-15.1.3.tar.gz')},
+                     'nova-16.0.1.tar.gz')},
+    'nova-base-plugin-blazar': {
+        'type': 'url',
+        'location': ('$tarballs_base/blazar-nova/'
+                     'blazar-nova-0.3.0.tar.gz')},
+    'novajoin-base': {
+        'type': 'url',
+        'location': ('$tarballs_base/novajoin/'
+                     'novajoin-1.0.17.tar.gz')},
     'octavia-base': {
         'type': 'url',
         'location': ('$tarballs_base/octavia/'
-                     'octavia-0.10.0.tar.gz')},
+                     'octavia-1.0.1.tar.gz')},
+    'opendaylight': {
+        'type': 'url',
+        'location': ('https://nexus.opendaylight.org/content/'
+                     'repositories/opendaylight.release/org/opendaylight/'
+                     'integration/distribution-karaf/0.6.1-Carbon/'
+                     'distribution-karaf-0.6.1-Carbon.tar.gz')},
     'panko-base': {
         'type': 'url',
         'location': ('$tarballs_base/panko/'
-                     'panko-2.0.3.tar.gz')},
+                     'panko-3.0.0.tar.gz')},
     'rally': {
         'type': 'url',
         'location': ('$tarballs_base/rally/'
-                     'rally-0.8.1.tar.gz')},
+                     'rally-master.tar.gz')},
     'sahara-base': {
         'type': 'url',
         'location': ('$tarballs_base/sahara/'
-                     'sahara-6.0.3.tar.gz')},
+                     'sahara-7.0.0.tar.gz')},
     'searchlight-base': {
         'type': 'url',
         'location': ('$tarballs_base/searchlight/'
-                     'searchlight-2.0.0.tar.gz')},
+                     'searchlight-3.0.0.tar.gz')},
     'senlin-base': {
         'type': 'url',
         'location': ('$tarballs_base/senlin/'
-                     'senlin-3.0.1.tar.gz')},
+                     'senlin-4.0.0.tar.gz')},
     'solum-base': {
         'type': 'url',
         'location': ('$tarballs_base/solum/'
-                     'solum-5.2.0.tar.gz')},
+                     'solum-5.4.0.tar.gz')},
     'swift-base': {
         'type': 'url',
         'location': ('$tarballs_base/swift/'
-                     'swift-2.13.1.tar.gz')},
-    'tacker': {
+                     'swift-2.15.1.tar.gz')},
+    'tacker-base': {
         'type': 'url',
         'location': ('$tarballs_base/tacker/'
-                     'tacker-0.7.0.tar.gz')},
+                     'tacker-0.8.0.tar.gz')},
     'tempest': {
         'type': 'url',
         'location': ('$tarballs_base/tempest/'
-                     'tempest-15.0.0.tar.gz')},
+                     'tempest-master.tar.gz')},
     'trove-base': {
         'type': 'url',
         'location': ('$tarballs_base/trove/'
-                     'trove-7.0.0.tar.gz')},
+                     'trove-8.0.0.tar.gz')},
     'vitrage-base': {
         'type': 'url',
         'location': ('$tarballs_base/vitrage/'
-                     'vitrage-1.5.2.tar.gz')},
+                     'vitrage-1.8.1.tar.gz')},
+    'vmtp': {
+        'type': 'url',
+        'location': ('$tarballs_base/vmtp/'
+                     'vmtp-2.4.4.tar.gz')},
     'watcher-base': {
         'type': 'url',
         'location': ('$tarballs_base/watcher/'
-                     'python-watcher-1.0.1.tar.gz')},
+                     'python-watcher-1.4.1.tar.gz')},
     'zaqar': {
         'type': 'url',
         'location': ('$tarballs_base/zaqar/'
-                     'zaqar-4.0.0.tar.gz')},
+                     'zaqar-5.0.0.tar.gz')},
     'zun-base': {
         'type': 'url',
         'location': ('$tarballs_base/zun/'
-                     'zun-master.tar.gz')}
+                     'zun-0.2.1.tar.gz')}
 }
 
 
 # NOTE(SamYaple): Only increment the UID. Never reuse old or removed UIDs.
-#     Starting point 42400+ was choosen arbitrarily to ensure no conflicts
+#     Starting point 42400+ was chosen arbitrarily to ensure no conflicts
 USERS = {
     'kolla-user': {
         'uid': 42400,
@@ -610,10 +730,6 @@ USERS = {
         'uid': 42418,
         'gid': 42418,
     },
-    'heka-user': {
-        'uid': 42419,
-        'gid': 42419,
-    },
     'horizon-user': {
         'uid': 42420,
         'gid': 42420,
@@ -770,6 +886,62 @@ USERS = {
         'uid': 42460,
         'gid': 42460,
     },
+    'ironic-inspector-user': {
+        'uid': 42461,
+        'gid': 42461,
+    },
+    'odl-user': {
+        'uid': 42462,
+        'gid': 42462,
+    },
+    'zun-user': {
+        'uid': 42463,
+        'gid': 42463,
+    },
+    'dragonflow-user': {
+        'uid': 42464,
+        'gid': 42464,
+    },
+    'qdrouterd-user': {
+        'uid': 42465,
+        'gid': 42465,
+    },
+    'ec2api-user': {
+        'uid': 42466,
+        'gid': 42466,
+    },
+    'sensu-user': {
+        'uid': 42467,
+        'gid': 42467,
+    },
+    'skydive-user': {
+        'uid': 42468,
+        'gid': 42468,
+    },
+    'kuryr-user': {
+        'uid': 42469,
+        'gid': 42469,
+    },
+    'novajoin-user': {
+        'uid': 42470,
+        'gid': 42470,
+    },
+    'blazar-user': {
+        'uid': 42471,
+        'gid': 42471,
+    },
+    'prometheus-user': {
+        'uid': 42472,
+        'gid': 42472,
+    },
+    'libvirt-user': {
+        'uid': 42473,  # unused user, but we need the group for socket access
+        'gid': 42473,
+    },
+    'fluentd-user': {
+        'uid': 42474,
+        'gid': 42474,
+    }
 }
 
 
@@ -786,8 +958,8 @@ def get_source_opts(type_=None, location=None, reference=None):
 
 def get_user_opts(uid, gid):
     return [
-        cfg.StrOpt('uid', default=uid, help='The user id'),
-        cfg.StrOpt('gid', default=gid, help='The group id'),
+        cfg.IntOpt('uid', default=uid, help='The user id'),
+        cfg.IntOpt('gid', default=gid, help='The group id'),
     ]
 
 
@@ -834,7 +1006,7 @@ def parse(conf, args, usage=None, prog=None,
 
     # NOTE(jeffrey4l): set the default base tag based on the
     # base option
-    conf.set_default('base_tag', DISTRO_RELEASE.get(conf.base))
+    conf.set_default('base_tag', DEFAULT_BASE_TAGS.get(conf.base))
 
     if not conf.base_image:
         conf.base_image = conf.base
