@@ -20,7 +20,6 @@ import errno
 import json
 import logging
 import os
-import pprint
 import re
 import requests
 import shutil
@@ -30,7 +29,6 @@ import tempfile
 import threading
 import time
 
-from distutils.version import StrictVersion
 import docker
 import git
 import jinja2
@@ -61,7 +59,7 @@ def make_a_logger(conf=None, image_name=None):
         log = logging.getLogger(__name__)
     if not log.handlers:
         if conf is None or not conf.logs_dir or not image_name:
-            handler = logging.StreamHandler(sys.stdout)
+            handler = logging.StreamHandler(sys.stderr)
             log.propagate = False
         else:
             filename = os.path.join(conf.logs_dir, "%s.log" % image_name)
@@ -89,12 +87,17 @@ STATUS_BUILDING = 'building'
 STATUS_UNMATCHED = 'unmatched'
 STATUS_MATCHED = 'matched'
 STATUS_UNPROCESSED = 'unprocessed'
+STATUS_SKIPPED = 'skipped'
 
 # All error status constants.
 STATUS_ERRORS = (STATUS_CONNECTION_ERROR, STATUS_PUSH_ERROR,
                  STATUS_ERROR, STATUS_PARENT_ERROR)
 
 
+class ArchivingError(Exception):
+    pass
+
+
 @contextlib.contextmanager
 def join_many(threads):
     try:
@@ -129,14 +132,14 @@ class DockerTask(task.Task):
         if self._dc is not None:
             return self._dc
         docker_kwargs = self.docker_kwargs.copy()
-        self._dc = docker.Client(version='auto', **docker_kwargs)
+        self._dc = docker.APIClient(version='auto', **docker_kwargs)
         return self._dc
 
 
 class Image(object):
     def __init__(self, name, canonical_name, path, parent_name='',
                  status=STATUS_UNPROCESSED, parent=None,
-                 source=None, logger=None):
+                 source=None, logger=None, docker_client=None):
         self.name = name
         self.canonical_name = canonical_name
         self.path = path
@@ -149,6 +152,8 @@ class Image(object):
         self.logger = logger
         self.children = []
         self.plugins = []
+        self.additions = []
+        self.dc = docker_client
 
     def copy(self):
         c = Image(self.name, self.canonical_name, self.path,
@@ -160,8 +165,13 @@ class Image(object):
             c.children = list(self.children)
         if self.plugins:
             c.plugins = list(self.plugins)
+        if self.additions:
+            c.additions = list(self.additions)
         return c
 
+    def in_docker_cache(self):
+        return len(self.dc.images(name=self.canonical_name, quiet=True)) == 1
+
     def __repr__(self):
         return ("Image(%s, %s, %s, parent_name=%s,"
                 " status=%s, parent=%s, source=%s)") % (
@@ -222,15 +232,9 @@ class PushTask(DockerTask):
                 self.success = False
 
     def push_image(self, image):
-        kwargs = dict(stream=True)
-
-        # Since docker 3.0.0, the argument of 'insecure_registry' is removed.
-        # To be compatible, set 'insecure_registry=True' for old releases.
-        dc_running_ver = StrictVersion(docker.version)
-        if dc_running_ver < StrictVersion('3.0.0'):
-            kwargs['insecure_registry'] = True
-
-        for response in self.dc.push(image.canonical_name, **kwargs):
+        for response in self.dc.push(image.canonical_name,
+                                     stream=True,
+                                     insecure_registry=True):
             stream = json.loads(response)
             if 'stream' in stream:
                 self.logger.info(stream['stream'])
@@ -257,7 +261,7 @@ class BuildTask(DockerTask):
 
     def run(self):
         self.builder(self.image)
-        if self.image.status == STATUS_BUILT:
+        if self.image.status in (STATUS_BUILT, STATUS_SKIPPED):
             self.success = True
 
     @property
@@ -305,6 +309,11 @@ class BuildTask(DockerTask):
         elif source.get('type') == 'git':
             clone_dir = '{}-{}'.format(dest_archive,
                                        source['reference'].replace('/', '-'))
+            if os.path.exists(clone_dir):
+                self.logger.info("Clone dir %s exists. Removing it.",
+                                 clone_dir)
+                shutil.rmtree(clone_dir)
+
             try:
                 self.logger.debug("Cloning from %s", source['source'])
                 git.Git().clone(source['source'], clone_dir)
@@ -361,7 +370,45 @@ class BuildTask(DockerTask):
         return buildargs
 
     def builder(self, image):
+
+        def make_an_archive(items, arcname, item_child_path=None):
+            if not item_child_path:
+                item_child_path = arcname
+            archives = list()
+            items_path = os.path.join(image.path, item_child_path)
+            for item in items:
+                archive_path = self.process_source(image, item)
+                if image.status in STATUS_ERRORS:
+                    raise ArchivingError
+                archives.append(archive_path)
+            if archives:
+                for archive in archives:
+                    with tarfile.open(archive, 'r') as archive_tar:
+                        archive_tar.extractall(path=items_path)
+            else:
+                try:
+                    os.mkdir(items_path)
+                except OSError as e:
+                    if e.errno == errno.EEXIST:
+                        self.logger.info(
+                            'Directory %s already exist. Skipping.',
+                            items_path)
+                    else:
+                        self.logger.error('Failed to create directory %s: %s',
+                                          items_path, e)
+                        image.status = STATUS_CONNECTION_ERROR
+                        raise ArchivingError
+            arc_path = os.path.join(image.path, '%s-archive' % arcname)
+            with tarfile.open(arc_path, 'w') as tar:
+                tar.add(items_path, arcname=arcname)
+            return len(os.listdir(items_path))
+
         self.logger.debug('Processing')
+
+        if image.status == STATUS_SKIPPED:
+            self.logger.info('Skipping %s' % image.name)
+            return
+
         if image.status == STATUS_UNMATCHED:
             return
 
@@ -380,32 +427,27 @@ class BuildTask(DockerTask):
             if image.status in STATUS_ERRORS:
                 return
 
-        plugin_archives = list()
-        plugins_path = os.path.join(image.path, 'plugins')
-        for plugin in image.plugins:
-            archive_path = self.process_source(image, plugin)
-            if image.status in STATUS_ERRORS:
-                return
-            plugin_archives.append(archive_path)
-        if plugin_archives:
-            for plugin_archive in plugin_archives:
-                with tarfile.open(plugin_archive, 'r') as plugin_archive_tar:
-                    plugin_archive_tar.extractall(path=plugins_path)
-        else:
+        if self.conf.install_type == 'source':
             try:
-                os.mkdir(plugins_path)
-            except OSError as e:
-                if e.errno == errno.EEXIST:
-                    self.logger.info('Directory %s already exist. Skipping.',
-                                     plugins_path)
-                else:
-                    self.logger.error('Failed to create directory %s: %s',
-                                      plugins_path, e)
-                    image.status = STATUS_CONNECTION_ERROR
-                    return
-        with tarfile.open(os.path.join(image.path, 'plugins-archive'),
-                          'w') as tar:
-            tar.add(plugins_path, arcname='plugins')
+                plugins_am = make_an_archive(image.plugins, 'plugins')
+            except ArchivingError:
+                self.logger.error(
+                    "Failed turning any plugins into a plugins archive")
+                return
+            else:
+                self.logger.debug(
+                    "Turned %s plugins into plugins archive",
+                    plugins_am)
+            try:
+                additions_am = make_an_archive(image.additions, 'additions')
+            except ArchivingError:
+                self.logger.error(
+                    "Failed turning any additions into a additions archive")
+                return
+            else:
+                self.logger.debug(
+                    "Turned %s additions into additions archive",
+                    additions_am)
 
         # Pull the latest image for the base distro only
         pull = self.conf.pull if image.parent is None else False
@@ -465,12 +507,8 @@ class WorkerThread(threading.Thread):
                 for attempt in six.moves.range(self.conf.retries + 1):
                     if self.should_stop:
                         break
-                    if attempt > 0:
-                        LOG.info("Attempting to run task %s for the %s time",
-                                 task.name, attempt + 1)
-                    else:
-                        LOG.info("Attempting to run task %s for the first"
-                                 " time", task.name)
+                    LOG.info("Attempt number: %s to run task: %s ",
+                             attempt + 1, task.name)
                     try:
                         task.run()
                         if task.success:
@@ -503,6 +541,7 @@ class KollaWorker(object):
         self.base_tag = conf.base_tag
         self.install_type = conf.install_type
         self.tag = conf.tag
+        self.base_arch = conf.base_arch
         self.images = list()
         rpm_setup_config = ([repo_file for repo_file in
                              conf.rpm_setup_config if repo_file is not None])
@@ -540,13 +579,22 @@ class KollaWorker(object):
         self.image_statuses_bad = dict()
         self.image_statuses_good = dict()
         self.image_statuses_unmatched = dict()
+        self.image_statuses_skipped = dict()
         self.maintainer = conf.maintainer
 
+        docker_kwargs = docker.utils.kwargs_from_env()
+        self.dc = docker.APIClient(version='auto', **docker_kwargs)
+
     def _get_images_dir(self):
         possible_paths = (
             PROJECT_ROOT,
             os.path.join(sys.prefix, 'share/kolla'),
-            os.path.join(sys.prefix, 'local/share/kolla'))
+            os.path.join(sys.prefix, 'local/share/kolla'),
+            # NOTE(zioproto): When Kolla is used within a snap, the env var
+            #                 $SNAP is the directory where the snap is mounted.
+            #                 https://github.com/zioproto/snap-kolla
+            #                 More info in snap packages https://snapcraft.io
+            os.path.join(os.environ.get('SNAP', ''), 'share/kolla'))
 
         for path in possible_paths:
             image_path = os.path.join(path, 'docker')
@@ -564,6 +612,7 @@ class KollaWorker(object):
         """Generates a list of docker commands based on provided configuration.
 
         :param rpm_setup_config: A list of .rpm or .repo paths or URLs
+                                 (can be empty)
         :return: A list of docker commands
         """
         rpm_setup = list()
@@ -581,6 +630,8 @@ class KollaWorker(object):
                 else:
                     # Copy .repo file from filesystem
                     cmd = "COPY {} /etc/yum.repos.d/".format(config)
+            elif not config:
+                cmd = ''
             else:
                 raise exception.KollaRpmSetupUnknownConfig(
                     'RPM setup must be provided as .rpm or .repo files.'
@@ -604,13 +655,31 @@ class KollaWorker(object):
                 os.path.join(self.working_dir, "base", "apt_preferences")
             )
 
+    def copy_dir(self, src, dest):
+        if not os.path.isdir(dest):
+            shutil.copytree(src, dest)
+        else:
+            for file in os.listdir(src):
+                src_path = os.path.join(src, file)
+                dest_path = os.path.join(dest, file)
+                if os.path.isdir(src_path):
+                    self.copy_dir(src_path, dest_path)
+                else:
+                    shutil.copy2(src_path, dest_path)
+
     def setup_working_dir(self):
-        """Creates a working directory for use while building"""
-        ts = time.time()
-        ts = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d_%H-%M-%S_')
-        self.temp_dir = tempfile.mkdtemp(prefix='kolla-' + ts)
-        self.working_dir = os.path.join(self.temp_dir, 'docker')
-        shutil.copytree(self.images_dir, self.working_dir)
+        """Creates a working directory for use while building."""
+        if self.conf.work_dir:
+            self.working_dir = os.path.join(self.conf.work_dir, 'docker')
+        else:
+            ts = time.time()
+            ts = datetime.datetime.fromtimestamp(ts).strftime(
+                '%Y-%m-%d_%H-%M-%S_')
+            self.temp_dir = tempfile.mkdtemp(prefix='kolla-' + ts)
+            self.working_dir = os.path.join(self.temp_dir, 'docker')
+        self.copy_dir(self.images_dir, self.working_dir)
+        for dir in self.conf.docker_dir:
+            self.copy_dir(dir, self.working_dir)
         self.copy_apt_files()
         LOG.debug('Created working dir: %s', self.working_dir)
 
@@ -629,7 +698,7 @@ class KollaWorker(object):
         return filters
 
     def _get_methods(self):
-        """Mapping of available Jinja methods
+        """Mapping of available Jinja methods.
 
         return a dictionary that maps available function names and their
         corresponding python methods to make them available in jinja templates
@@ -660,9 +729,13 @@ class KollaWorker(object):
         for path in self.docker_build_paths:
             template_name = "Dockerfile.j2"
             image_name = path.split("/")[-1]
+            ts = time.time()
+            build_date = datetime.datetime.fromtimestamp(ts).strftime(
+                '%Y%m%d')
             values = {'base_distro': self.base,
                       'base_image': self.conf.base_image,
                       'base_distro_tag': self.base_tag,
+                      'base_arch': self.base_arch,
                       'supported_distro_release': supported_distro_release,
                       'install_metatype': self.install_metatype,
                       'image_prefix': self.image_prefix,
@@ -673,7 +746,8 @@ class KollaWorker(object):
                       'kolla_version': kolla_version,
                       'image_name': image_name,
                       'users': self.get_users(),
-                      'rpm_setup': self.rpm_setup}
+                      'rpm_setup': self.rpm_setup,
+                      'build_date': build_date}
             env = jinja2.Environment(  # nosec: not used to render HTML
                 loader=jinja2.FileSystemLoader(self.working_dir))
             env.filters.update(self._get_filters())
@@ -713,7 +787,7 @@ class KollaWorker(object):
         return {tpl_name: tpl_content}
 
     def find_dockerfiles(self):
-        """Recursive search for Dockerfiles in the working directory"""
+        """Recursive search for Dockerfiles in the working directory."""
         self.docker_build_paths = list()
         path = self.working_dir
         filename = 'Dockerfile.j2'
@@ -726,11 +800,12 @@ class KollaWorker(object):
         LOG.debug('Found %d Dockerfiles', len(self.docker_build_paths))
 
     def cleanup(self):
-        """Remove temp files"""
-        shutil.rmtree(self.temp_dir)
+        """Remove temp files."""
+        if not self.conf.work_dir:
+            shutil.rmtree(self.temp_dir)
 
     def filter_images(self):
-        """Filter which images to build"""
+        """Filter which images to build."""
         filter_ = list()
 
         if self.regex:
@@ -750,14 +825,21 @@ class KollaWorker(object):
         if filter_:
             patterns = re.compile(r"|".join(filter_).join('()'))
             for image in self.images:
-                if image.status == STATUS_MATCHED:
+                if image.status in (STATUS_MATCHED, STATUS_SKIPPED):
                     continue
                 if re.search(patterns, image.name):
                     image.status = STATUS_MATCHED
                     while (image.parent is not None and
-                           image.parent.status != STATUS_MATCHED):
+                           image.parent.status not in (STATUS_MATCHED,
+                                                       STATUS_SKIPPED)):
                         image = image.parent
-                        image.status = STATUS_MATCHED
+                        if self.conf.skip_parents:
+                            image.status = STATUS_SKIPPED
+                        elif (self.conf.skip_existing and
+                              image.in_docker_cache()):
+                            image.status = STATUS_SKIPPED
+                        else:
+                            image.status = STATUS_MATCHED
                         LOG.debug('Image %s matched regex', image.name)
                 else:
                     image.status = STATUS_UNMATCHED
@@ -766,7 +848,7 @@ class KollaWorker(object):
                 image.status = STATUS_MATCHED
 
     def summary(self):
-        """Walk the dictionary of images statuses and print results"""
+        """Walk the dictionary of images statuses and print results."""
         # For debug we print the logs again if the image error'd. This is to
         # help us debug and it will be extra helpful in the gate.
         for image in self.images:
@@ -778,13 +860,14 @@ class KollaWorker(object):
             'built': [],
             'failed': [],
             'not_matched': [],
+            'skipped': [],
         }
 
         if self.image_statuses_good:
             LOG.info("=========================")
             LOG.info("Successfully built images")
             LOG.info("=========================")
-            for name in self.image_statuses_good.keys():
+            for name in sorted(self.image_statuses_good.keys()):
                 LOG.info(name)
                 results['built'].append({
                     'name': name,
@@ -794,46 +877,57 @@ class KollaWorker(object):
             LOG.info("===========================")
             LOG.info("Images that failed to build")
             LOG.info("===========================")
-            for name, status in self.image_statuses_bad.items():
+            for name, status in sorted(self.image_statuses_bad.items()):
                 LOG.error('%s Failed with status: %s', name, status)
                 results['failed'].append({
                     'name': name,
                     'status': status,
                 })
-                if self.conf.logs_dir and status == STATUS_ERROR:
-                    os.symlink("%s.log" % name,
-                               os.path.join(self.conf.logs_dir,
-                                            "000_FAILED_%s.log" % name))
 
         if self.image_statuses_unmatched:
             LOG.debug("=====================================")
             LOG.debug("Images not matched for build by regex")
             LOG.debug("=====================================")
-            for name in self.image_statuses_unmatched.keys():
+            for name in sorted(self.image_statuses_unmatched.keys()):
                 LOG.debug(name)
                 results['not_matched'].append({
                     'name': name,
                 })
 
+        if self.image_statuses_skipped:
+            LOG.debug("================================")
+            LOG.debug("Images skipped due build options")
+            LOG.debug("================================")
+            for name in sorted(self.image_statuses_skipped.keys()):
+                LOG.debug(name)
+                results['skipped'].append({
+                    'name': name,
+                })
+
         return results
 
     def get_image_statuses(self):
         if any([self.image_statuses_bad,
                 self.image_statuses_good,
-                self.image_statuses_unmatched]):
+                self.image_statuses_unmatched,
+                self.image_statuses_skipped]):
             return (self.image_statuses_bad,
                     self.image_statuses_good,
-                    self.image_statuses_unmatched)
+                    self.image_statuses_unmatched,
+                    self.image_statuses_skipped)
         for image in self.images:
             if image.status == STATUS_BUILT:
                 self.image_statuses_good[image.name] = image.status
             elif image.status == STATUS_UNMATCHED:
                 self.image_statuses_unmatched[image.name] = image.status
+            elif image.status == STATUS_SKIPPED:
+                self.image_statuses_skipped[image.name] = image.status
             else:
                 self.image_statuses_bad[image.name] = image.status
         return (self.image_statuses_bad,
                 self.image_statuses_good,
-                self.image_statuses_unmatched)
+                self.image_statuses_unmatched,
+                self.image_statuses_skipped)
 
     def build_image_list(self):
         def process_source_installation(image, section):
@@ -871,7 +965,8 @@ class KollaWorker(object):
             del match
             image = Image(image_name, canonical_name, path,
                           parent_name=parent_name,
-                          logger=make_a_logger(self.conf, image_name))
+                          logger=make_a_logger(self.conf, image_name),
+                          docker_client=self.dc)
 
             if self.install_type == 'source':
                 # NOTE(jeffrey4l): register the opts if the section didn't
@@ -894,6 +989,20 @@ class KollaWorker(object):
                                   plugin)
                     image.plugins.append(
                         process_source_installation(image, plugin))
+                for addition in [
+                    match.group(0) for match in
+                    (re.search('^{}-additions-.+'.format(image.name),
+                     section) for section in all_sections) if match]:
+                    try:
+                        self.conf.register_opts(
+                            common_config.get_source_opts(),
+                            addition
+                        )
+                    except cfg.DuplicateOptError:
+                        LOG.debug('Addition %s already registered in config',
+                                  addition)
+                    image.additions.append(
+                        process_source_installation(image, addition))
 
             self.images.append(image)
 
@@ -946,10 +1055,10 @@ class KollaWorker(object):
 
         ancestry = {base.name: []}
         list_children(base.children, ancestry)
-        pprint.pprint(ancestry)
+        json.dump(ancestry, sys.stdout, indent=2)
 
     def find_parents(self):
-        """Associate all images with parents and children"""
+        """Associate all images with parents and children."""
         sort_images = dict()
 
         for image in self.images:
@@ -962,7 +1071,7 @@ class KollaWorker(object):
                     image.parent = parent
 
     def build_queue(self, push_queue):
-        """Organizes Queue list
+        """Organizes Queue list.
 
         Return a list of Queues that have been organized into a hierarchy
         based on dependencies
