@@ -14,10 +14,29 @@ function setup_disk {
 
     if [ ! -f /docker ]; then
         sudo dd if=/dev/zero of=/docker bs=1M count=20480
-        losetup -f /docker
+        sudo losetup -f /docker
         DEV=$(losetup -a | awk -F: '/\/docker/ {print $1}')
     fi
 
+# Excerpts from https://github.com/openstack-infra/devstack-gate/blob/dc49f9e6eb18e42c6b175e4e146fa8f3b7633279/functions.sh#L306
+    if [ -b /dev/xvde ]; then
+        DEV2='/dev/xvde'
+        if mount | grep ${DEV2} > /dev/null; then
+            echo "*** ${DEV2} appears to already be mounted"
+            echo "*** ${DEV2} unmounting and reformating"
+            sudo umount ${DEV2}
+        fi
+        sudo parted ${DEV2} --script -- mklabel msdos
+        sync
+        sudo partprobe
+        sudo mkfs.ext4 ${DEV2}
+        sudo mount ${DEV2} /mnt
+        sudo find /opt/ -mindepth 1 -maxdepth 1 -exec mv {} /mnt/ \;
+        sudo umount /mnt
+        sudo mount ${DEV2} /opt
+        grep -q ${DEV2} /proc/mounts || exit 1
+    fi
+
     # Format Disks and setup Docker to use BTRFS
     sudo parted ${DEV} -s -- mklabel msdos
     sudo rm -rf /var/lib/docker
@@ -35,16 +54,14 @@ function setup_disk {
 # (SamYaple)TODO: Remove the path overriding
 export PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
 
-sudo tee /etc/yum.repos.d/docker.repo << EOF
-[docker]
-name=Docker Main Repository
-baseurl=https://yum.dockerproject.org/repo/main/centos/7
-enabled=1
-gpgcheck=1
-gpgkey=https://yum.dockerproject.org/gpg
-EOF
+sudo yum-config-manager \
+    --add-repo \
+    https://download.docker.com/linux/centos/docker-ce.repo
 
-sudo yum -y install libffi-devel openssl-devel docker-engine-1.12.6 docker-engine-selinux-1.12.6 btrfs-progs
+sudo yum -y install libffi-devel openssl-devel docker-ce btrfs-progs
+
+# Disable SELinux
+setenforce 0
 
 setup_disk
 
@@ -53,7 +70,7 @@ sudo mkdir /etc/systemd/system/docker.service.d
 sudo tee /etc/systemd/system/docker.service.d/kolla.conf << EOF
 [Service]
 ExecStart=
-ExecStart=/usr/bin/dockerd --storage-driver btrfs
+ExecStart=/usr/bin/dockerd --storage-driver btrfs --insecure-registry=0.0.0.0/0
 MountFlags=shared
 EOF
 
@@ -62,4 +79,8 @@ sudo systemctl daemon-reload
 sudo systemctl start docker
 sudo docker info
 
+# disable ipv6 until we're sure routes to fedora mirrors work properly
+sudo sh -c 'echo "net.ipv6.conf.all.disable_ipv6 = 1" > /etc/sysctl.d/disable_ipv6.conf'
+sudo /usr/sbin/sysctl -p
+
 echo "Completed $0."
